@@ -23,16 +23,19 @@
 # SPDX-License-Identifier: LGPL-2.1-only
 #
 
+import gi
 import os
 import locale
 import tempfile
 
-from gi.repository import GLib, GObject, Gio, Gdk, Gtk, Pango, Adw
+gi.require_version('CambalachePrivate', '4.0')
+from gi.repository import GLib, GObject, Gio, Gdk, Gtk, Pango, Adw, GtkSource, CambalachePrivate
 from .cmb_tutor import CmbTutor, CmbTutorState
 from . import cmb_tutorial
 
 from cambalache import (
     CmbProject,
+    CmbBaseFileMonitor,
     CmbUI,
     CmbCSS,
     CmbObject,
@@ -52,6 +55,7 @@ from cambalache.cmb_blueprint import CmbBlueprintError
 logger = getLogger(__name__)
 
 GObject.type_ensure(CmbGResourceEditor.__gtype__)
+GObject.type_ensure(CambalachePrivate.Svg.__gtype__)
 
 
 @Gtk.Template(resource_path="/ar/xjuan/Cambalache/app/cmb_window.ui")
@@ -81,6 +85,7 @@ class CmbWindow(Adw.ApplicationWindow):
     stack = Gtk.Template.Child()
 
     # Start screen
+    logo = Gtk.Template.Child()
     version_label = Gtk.Template.Child()
     front_notification_list_view = Gtk.Template.Child()
     notification_dialog = Gtk.Template.Child()
@@ -106,13 +111,7 @@ class CmbWindow(Adw.ApplicationWindow):
     type_chooser = Gtk.Template.Child()
     editor_stack = Gtk.Template.Child()
     ui_editor = Gtk.Template.Child()
-    ui_requires_editor = Gtk.Template.Child()
-    ui_fragment_editor = Gtk.Template.Child()
-    fragment_editor = Gtk.Template.Child()
     object_editor = Gtk.Template.Child()
-    object_layout_editor = Gtk.Template.Child()
-    accessible_editor = Gtk.Template.Child()
-    signal_editor = Gtk.Template.Child()
     css_editor = Gtk.Template.Child()
     gresource_editor = Gtk.Template.Child()
 
@@ -120,6 +119,10 @@ class CmbWindow(Adw.ApplicationWindow):
     intro_button = Gtk.Template.Child()
     menu_button = Gtk.Template.Child()
 
+    # Properties
+    current_file_object = GObject.Property(type=CmbBaseFileMonitor, flags=GObject.ParamFlags.READWRITE)
+    source_style = GObject.Property(type=GtkSource.StyleScheme, flags=GObject.ParamFlags.READWRITE)
+
     # Settings
     completed_intro = GObject.Property(type=bool, default=False, flags=GObject.ParamFlags.READWRITE)
 
@@ -134,10 +137,19 @@ class CmbWindow(Adw.ApplicationWindow):
     def __init__(self, **kwargs):
         self.__project = None
         self.__last_saved_index = None
+        self.__last_saved_index_version = None
         self.__np_location = None
 
         super().__init__(**kwargs)
 
+        # Logo animation
+        def toggle_animation(gesture, n_press, x, y):
+            self.logo.props.paintable.props.state = 0 if self.logo.props.paintable.props.state else 2
+
+        click = Gtk.GestureClick()
+        click.connect("released", toggle_animation)
+        self.logo.add_controller(click)
+
         self.gtk4_import_filters = Gio.ListStore()
 
         for filter in [
@@ -185,6 +197,7 @@ class CmbWindow(Adw.ApplicationWindow):
             "new",
             "notification",
             "open",
+            "open_inspector",
             "paste",
             "patreon",
             "redo",
@@ -302,6 +315,10 @@ class CmbWindow(Adw.ApplicationWindow):
         self.__load_window_state()
         self.__update_actions()
 
+        self.source_style_manager = GtkSource.StyleSchemeManager.get_default()
+        app.props.style_manager.connect("notify::dark", lambda o, p: self.__update_dark_mode(app.props.style_manager))
+        self.__update_dark_mode(app.props.style_manager)
+
         # Bind preview
         hide_placeholders_button = Gtk.ToggleButton(tooltip_text=_("Hide placeholders"), icon_name="view-conceal-symbolic")
         self.type_chooser.content.append(hide_placeholders_button)
@@ -357,13 +374,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
         # Clear Editors
         self.ui_editor.object = None
-        self.ui_requires_editor.object = None
-        self.ui_fragment_editor.object = None
         self.object_editor.object = None
-        self.object_layout_editor.object = None
-        self.accessible_editor.object = None
-        self.signal_editor.object = None
-        self.fragment_editor.object = None
 
         if project:
             self.__project.connect("notify::filename", self.__on_project_filename_notify)
@@ -380,7 +391,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
     def __update_window_title(self):
         if self.project is None:
-            self.title.props.title = _("Cambalache")
+            self.title.props.title = "Cambalache"
             self.title.props.subtitle = None
             return
 
@@ -472,6 +483,20 @@ class CmbWindow(Adw.ApplicationWindow):
         self.np_ui_entry.set_sensitive(sensitive)
         self.__update_action_new()
 
+    def __update_dark_mode(self, style_manager):
+        if style_manager.props.dark:
+            self.source_style = self.source_style_manager.get_scheme("Adwaita-dark")
+            paintable = CambalachePrivate.Svg(resource="/ar/xjuan/Cambalache/app/images/logo-dark.gpa")
+            self.add_css_class("dark")
+        else:
+            paintable = CambalachePrivate.Svg(resource="/ar/xjuan/Cambalache/app/images/logo.gpa")
+            self.remove_css_class("dark")
+            self.source_style = self.source_style_manager.get_scheme("tango")
+
+        paintable.props.playing = True
+        paintable.props.state = 2
+        self.logo.props.paintable = paintable
+
     def __np_name_to_ui(self, binding, value):
         if len(value):
             return value.lower().rsplit(".", 1)[0] + ".ui"
@@ -562,21 +587,23 @@ class CmbWindow(Adw.ApplicationWindow):
 
         if isinstance(obj, CmbUI):
             self.ui_editor.object = obj
-            self.ui_requires_editor.object = obj
-            self.ui_fragment_editor.object = obj
+            self.current_file_object = obj
             self.workspace_stack.set_visible_child_name("ui")
             self.editor_stack.set_visible_child_name("ui")
             obj = None
         elif isinstance(obj, CmbObject):
+            self.current_file_object = obj.ui
             self.workspace_stack.set_visible_child_name("ui")
             self.editor_stack.set_visible_child_name("object")
             if obj:
                 self.__user_message_by_type(obj.info)
         elif isinstance(obj, CmbCSS):
+            self.current_file_object = None
             self.css_editor.object = obj
             self.editor_stack.set_visible_child_name("css")
             obj = None
         elif isinstance(obj, CmbGResource):
+            self.current_file_object = obj
             self.gresource_editor.object = obj
             self.workspace_stack.set_visible_child_name("gresource")
             self.editor_stack.set_visible_child_name("gresource")
@@ -585,11 +612,6 @@ class CmbWindow(Adw.ApplicationWindow):
 
         self.object_editor.object = obj
 
-        is_not_builtin = not obj.info.is_builtin if obj else True
-        for editor in [self.object_layout_editor, self.signal_editor, self.fragment_editor, self.accessible_editor]:
-            editor.object = obj
-            editor.props.visible = is_not_builtin
-
         self.__update_action_add_object()
         self.__update_action_remove_parent()
 
@@ -629,10 +651,13 @@ class CmbWindow(Adw.ApplicationWindow):
         self.actions["remove_parent"].set_enabled(False)
 
     def __needs_saving(self):
-        has_project = self.__is_project_visible()
-        changed = has_project and self.project.history_index != self.__last_saved_index
+        if self.project is None:
+            return False
 
-        return has_project and changed
+        return self.__is_project_visible() and (
+            self.project.history_index != self.__last_saved_index or
+            self.project.history_index_version != self.__last_saved_index_version
+        )
 
     def __update_action_save(self):
         changed = self.__needs_saving()
@@ -658,6 +683,7 @@ class CmbWindow(Adw.ApplicationWindow):
             "add_gresource",
             "delete",
             "import",
+            "import_directory",
             "close",
             "debug"
         ]:
@@ -687,33 +713,50 @@ class CmbWindow(Adw.ApplicationWindow):
         return dialog
 
     def present_message_to_user(self, message, secondary_text=None, details=None):
+        # TODO: replace with custom widget
+
         dialog = Gtk.MessageDialog(
             transient_for=self,
-            message_type=Gtk.MessageType.INFO,
-            buttons=Gtk.ButtonsType.OK,
             text=message,
             secondary_text=secondary_text,
             modal=True,
         )
 
+        dialog.add_button(_("Copy to clipboard"), 1)
+        dialog.add_button(_("Close"), Gtk.ResponseType.CLOSE)
+
         if details:
+            sw = Gtk.ScrolledWindow(
+                vexpand=True,
+                propagate_natural_width=True,
+                propagate_natural_height=True,
+                max_content_width=800,
+                max_content_height=480,
+            )
             box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=4)
+            sw.set_child(box)
 
             for detail in details:
                 box.append(
                     Gtk.Label(
                         label=detail,
+                        use_markup=True,
                         halign=Gtk.Align.START,
                         xalign=0.0,
-                        lines=2,
-                        max_width_chars=80,
-                        wrap_mode=Pango.WrapMode.CHAR,
-                        ellipsize=Pango.EllipsizeMode.END,
+                        max_width_chars=120,
+                        wrap_mode=Pango.WrapMode.WORD,
                     )
                 )
-            dialog.props.message_area.append(box)
+            dialog.props.message_area.append(sw)
 
-        dialog.connect("response", lambda d, r: dialog.destroy())
+        def on_response(dialog, response):
+            if response == 1:
+                clip = self.get_clipboard()
+                clip.set(f"{message}\n{secondary_text}\n{'\n'.join(details or [])}")
+            else:
+                dialog.destroy()
+
+        dialog.connect("response", on_response)
         dialog.present()
 
     def __check_if_filename_is_in_portal(self, filename):
@@ -722,7 +765,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
         return filename.startswith(f"/run/user/{os.getuid()}/doc/")
 
-    def import_ui(self, filename, target_tk=None, autoselect=True):
+    def import_ui(self, filename, target_tk=None, autoselect=True, present_errors=True):
         if self.project is None:
             dirname = os.path.dirname(filename)
             basename = os.path.basename(filename)
@@ -733,7 +776,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
             if not target_tk:
                 self.ask_gtk_version(filename)
-                return
+                return (None, None)
 
             self.project = CmbProject(filename=os.path.join(dirname, f"{name}.cmb"), target_tk=target_tk)
             self.__set_page("workspace")
@@ -745,7 +788,7 @@ class CmbWindow(Adw.ApplicationWindow):
         if ui:
             if autoselect:
                 self.project.set_selection([ui])
-            return
+            return (None, None)
 
         try:
             if self.__check_if_filename_is_in_portal(filename):
@@ -762,8 +805,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
                 filename = os.path.basename(filename)
                 name, ext = os.path.splitext(filename)
-                unsupported_features_list = None
-                text = None
+                title = _("Error importing {filename}").format(filename=os.path.basename(filename))
 
                 if len(msg) > 1:
                     # Translators: This is used to create a unordered list of unsupported features to show the user
@@ -776,6 +818,10 @@ class CmbWindow(Adw.ApplicationWindow):
                     last_msg = _("Your file will be saved as '{name}.cmb.ui' to avoid data loss.").format(name=name)
 
                     unsupported_features_list = [first_msg] + list + [last_msg]
+                    if present_errors:
+                        self.present_message_to_user(title, details=unsupported_features_list)
+
+                    return (title, "\n".join(unsupported_features_list))
                 else:
                     unsupported_feature = msg[0]
                     text = _(
@@ -783,17 +829,24 @@ class CmbWindow(Adw.ApplicationWindow):
                         "Your file will be saved as '{name}.cmb.ui' to avoid data loss."
                     ).format(unsupported_feature=unsupported_feature, name=name)
 
-                self.present_message_to_user(
-                    _("Error importing {filename}").format(filename=os.path.basename(filename)),
-                    secondary_text=text,
-                    details=unsupported_features_list,
-                )
+                    if present_errors:
+                        self.present_message_to_user(title, secondary_text=text)
+
+                    return (title, text)
+
+            # All good!
+            return (None, None)
         except Exception as e:
             filename = os.path.basename(filename)
             logger.warning(f"Error loading {filename}", exc_info=True)
-            self.present_message_to_user(
-                _("Error importing {filename}").format(filename=filename), secondary_text=str(e)
-            )
+
+            title = _("Exception importing {filename}").format(filename=filename)
+            msg = str(e)
+
+            if present_errors:
+                self.present_message_to_user(title, secondary_text=msg)
+
+            return (title, msg)
 
     def ask_gtk_version(self, filename):
         basename = os.path.basename(filename)
@@ -823,6 +876,7 @@ class CmbWindow(Adw.ApplicationWindow):
 
         self.project = CmbProject(filename=filename, target_tk=target_tk)
         self.__last_saved_index = self.project.history_index
+        self.__last_saved_index_version = self.project.history_index_version
 
         # Create UI and select it
         ui = self.project.add_ui(uipath)
@@ -852,6 +906,7 @@ class CmbWindow(Adw.ApplicationWindow):
                 self.project = CmbProject(filename=filename, target_tk=target_tk)
 
             self.__last_saved_index = self.project.history_index
+            self.__last_saved_index_version = self.project.history_index_version
             self.__set_page("workspace")
             self.__update_actions()
         except Exception as e:
@@ -1125,16 +1180,18 @@ class CmbWindow(Adw.ApplicationWindow):
             details=unsupported_features_list,
         )
 
-    def import_file(self, path, autoselect=True):
+    def import_file(self, path, autoselect=True, present_errors=True):
         content_type = utils.content_type_guess(path)
 
         if content_type in ["application/x-gtk-builder", "application/x-glade", "text/x-blueprint"]:
-            self.import_ui(path, autoselect=autoselect)
+            return self.import_ui(path, autoselect=autoselect, present_errors=present_errors)
         elif content_type == "text/css":
             self.project.add_css(path)
         elif content_type == "application/xml" and path.endswith("gresource.xml"):
             self.project.import_gresource(path)
 
+        return (None, None)
+
     def _on_import_activate(self, action, data):
         if self.project is None:
             return
@@ -1205,16 +1262,30 @@ class CmbWindow(Adw.ApplicationWindow):
 
                 basedir = self.project.dirname + "/"
 
+                errors = []
+
                 for i, path in enumerate(files):
                     progressbar.set_text(path.removeprefix(basedir))
                     progressbar.set_fraction(i/n_files)
-                    self.import_file(path, autoselect=False)
+                    error, error_details = self.import_file(path, autoselect=False, present_errors=False)
+
+                    if error:
+                        errors.append(f"\n<b>{error}</b>\n{error_details}")
 
                     while main_loop.pending():
                         main_loop.iteration(False)
 
                 self.project.history_pop()
-                self._show_message(_("Imported {n} files").format(n=n_files))
+
+                if errors:
+                    text = _("{errors} files out of {n} had errors while loading").format(errors=len(errors), n=n_files)
+                    self.present_message_to_user(
+                        _("Error importing directory {basename}").format(basename=os.path.basename(dirpath)),
+                        secondary_text=text,
+                        details=errors
+                    )
+                else:
+                    self._show_message(_("Imported {n} files").format(n=n_files))
 
                 progress.close()
 
@@ -1250,6 +1321,7 @@ class CmbWindow(Adw.ApplicationWindow):
         finally:
             if retval:
                 self.__last_saved_index = self.project.history_index
+                self.__last_saved_index_version = self.project.history_index_version
                 self.__update_action_save()
                 self.emit("project-saved", self.project)
 
@@ -1514,6 +1586,9 @@ class CmbWindow(Adw.ApplicationWindow):
     def _on_inspect_activate(self, action, data):
         self.view.inspect()
 
+    def _on_open_inspector_activate(self, action, data):
+        self.view.set_interactive_debugging(True)
+
     def _on_open_recent_activate(self, action, data):
         self.__app_activate_open(data.get_string(), "")
 
