@@ -49,7 +49,7 @@ from .cmb_property_info import CmbPropertyInfo
 from .cmb_layout_property import CmbLayoutProperty
 from .cmb_library_info import CmbLibraryInfo
 from .cmb_type_info import CmbTypeInfo
-from .cmb_objects_base import CmbSignal
+from .cmb_base_objects import CmbSignal
 from .cmb_blueprint import cmb_blueprint_decompile, cmb_blueprint_compile
 from .utils import FileHash
 from . import constants, utils
@@ -147,7 +147,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 self.target_tk = target_tk
 
         if self.target_tk is None or self.target_tk == "":
-            raise Exception("Either target_tk or filename are required")
+            raise Exception(_("Either target_tk or filename are required"))
 
         # DataModel is only used internally
         self.db = CmbDB(target_tk=self.target_tk)
@@ -198,6 +198,11 @@ class CmbProject(GObject.Object, Gio.ListModel):
 
         self.db.set_data("history_index", value)
 
+    @GObject.Property(type=int)
+    def history_index_version(self):
+        row = self.db.execute("SELECT version FROM history WHERE history_id=?;", (self.history_index, )).fetchone()
+        return row[0] if row else 0
+
     def _get_table_data(self, table):
         c = self.db.cursor()
 
@@ -331,7 +336,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 root = etree.fromstring(content.text.encode())
                 ui_id = self.db.import_from_node(root, None)
             else:
-                raise Exception("content tag is missing")
+                raise Exception(_("content tag is missing"))
 
         msgs, detail_msg = self.__get_import_errors()
         self.db.errors = None
@@ -350,10 +355,15 @@ class CmbProject(GObject.Object, Gio.ListModel):
         if owner_id:
             for property in node.findall("property"):
                 property_id, type_id = utils.xml_node_get(property, "id", "type-id")
-                self.db.execute(
-                    "INSERT INTO property(owner_id, property_id, type_id) VALUES (?, ?, ?);",
-                    (owner_id, property_id, type_id)
-                )
+
+                try :
+                    self.db.execute(
+                        "INSERT INTO property(owner_id, property_id, type_id) VALUES (?, ?, ?);",
+                        (owner_id, property_id, type_id)
+                    )
+                except sqlite3.IntegrityError as e:
+                    logger.warning(f"Error inserting property {owner_id}::{property_id} of type {type_id}: {e}")
+                    continue
 
                 for key in [
                     "is_object",
@@ -390,18 +400,29 @@ class CmbProject(GObject.Object, Gio.ListModel):
         self.__populate_ui(ui_id)
 
     def __load_css_from_node(self, node):
-        css_filename, priority, is_global = utils.xml_node_get(node, ["filename", "priority", "is_global"])
-        css_id = self.db.add_css(css_filename, priority, is_global)
+        filename, sha256, priority, is_global = utils.xml_node_get(node, ["filename", "sha256", "priority", "is_global"])
 
-        # FIXME: Load UI relation this is for old format <= 0.95
-        for ui in node.getchildren():
-            if ui.tag != "ui":
-                continue
-            row = self.db.execute("SELECT ui_id FROM ui WHERE filename=?;", (ui.text,)).fetchone()
-            if row:
-                (ui_id,) = row
-                self.db.execute("INSERT INTO css_ui VALUES (?, ?)", (css_id, ui_id))
+        if filename:
+            fullpath, relpath = self.__get_abs_path(filename)
+            with open(fullpath) as fd:
+                css = fd.read()
 
+                m = hashlib.sha256()
+                m.update(css.encode())
+                hexdigest = m.hexdigest()
+
+                if sha256 != hexdigest:
+                    logger.warning(f"{filename} hash mismatch, file was modified")
+
+                fd.close()
+        else:
+            content = node.find("content")
+            if content:
+                css = content.text.encode()
+            else:
+                raise Exception(_("content tag is missing"))
+
+        css_id = self.db.add_css(filename, priority, is_global, css=css)
         self.__populate_css(css_id)
 
     def __load_gresource_from_node(self, node):
@@ -423,7 +444,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 root = etree.fromstring(content.text.encode())
                 gresource_id = self.db.import_gresource_from_node(root, None)
             else:
-                raise Exception("content tag is missing")
+                raise Exception(_("content tag is missing"))
 
         self.__populate_gresource(gresource_id)
 
@@ -439,7 +460,11 @@ class CmbProject(GObject.Object, Gio.ListModel):
         target_tk = root.get("target_tk", None)
 
         if target_tk != self.target_tk:
-            raise Exception(f"Can not load a {target_tk} target in {self.target_tk} project.")
+            raise Exception(
+                _("Can not load a {target} target in {project_target} project.").format(
+                    target=target_tk, project_target=self.target_tk
+                )
+            )
 
         version = root.get("version", None)
         version = (0, 0, 0) if version is None else utils.parse_version(version)
@@ -447,12 +472,14 @@ class CmbProject(GObject.Object, Gio.ListModel):
         if version > self.db.version:
             version = ".".join(map(str, version))
             raise Exception(
-                f"File format {version} is not supported by this release,\nplease update to a newer version to open this file."
+                _("File format {version} is not supported by this release,\n"
+                  "please update to a newer version to open this file.").format(version=version)
             )
 
         if version <= (0, 94, 0):
             raise Exception(
-                f"Project format {version} is not supported, Open/save with Cambalache 0.96.0 to migrate to the new format."
+                _("Project format {version} is not supported, "
+                  "Open/save with Cambalache 0.96.0 to migrate to the new format.").format(version=version)
             )
 
         ui_graph = {}
@@ -481,7 +508,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
             elif child.tag == "icontheme-search-path":
                 self.icontheme_search_paths.append(child.text)
             else:
-                raise Exception(f"Unknown tag {child.tag} in project file.")
+                raise Exception(_("Unknown tag {tag} in project file.").format(tag=child.tag))
 
         for node in css_list:
             self.__load_css_from_node(node)
@@ -498,7 +525,8 @@ class CmbProject(GObject.Object, Gio.ListModel):
             ts = TopologicalSorter(ui_node_graph)
             sorted_ui_nodes = tuple(ts.static_order())
         except CycleError as e:
-            raise Exception(f"Could not load project because of dependency cycle {e}")
+            logger.warning(f"Dependency cycle detected: {e}")
+            raise Exception(_("Could not load project because of dependency cycle"))
 
         # Load UI in topological order
         for node in sorted_ui_nodes:
@@ -538,7 +566,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
     def dirname(self):
         return os.path.dirname(self.__filename) if self.__filename else "."
 
-    def __save_xml_and_update_node(self, node, root, filename):
+    def __save_xml_and_update_node(self, node, root, filename, file_object):
         if root is None or filename is None:
             return
 
@@ -587,6 +615,9 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 with open(fullpath, "wb") as fd:
                     fd.write(blueprint_decompiled.encode())
             else:
+                if file_object:
+                    file_object.saving = True
+
                 # Dump xml to file
                 with open(fullpath, "wb") as fd:
                     hash_file = FileHash(fd)
@@ -604,6 +635,8 @@ class CmbProject(GObject.Object, Gio.ListModel):
         node.append(content)
 
     def __save_ui_and_get_node(self, ui_id, template_id, filename):
+        file_object = self.get_object_by_id(ui_id)
+
         ui = E.ui()
 
         # Get a list of types declared in the project used by this UI
@@ -667,7 +700,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
         # Save UI file
         if filename:
             root = self.db.export_ui(ui_id)
-            self.__save_xml_and_update_node(ui, root, filename)
+            self.__save_xml_and_update_node(ui, root, filename, file_object)
         else:
             # Embed UI content in project as CDATA
             root = self.db.export_ui(ui_id)
@@ -675,23 +708,47 @@ class CmbProject(GObject.Object, Gio.ListModel):
 
         return ui
 
-    def __save_css_and_get_node(self, css_id, filename, priority, is_global):
+    def __save_css_and_get_node(self, css_id, filename, css_text, priority, is_global):
+        file_object = self.get_css_by_id(css_id)
+        if file_object:
+            file_object.saving = True
+
         css = E.css()
 
-        utils.xml_node_set(css, "filename", filename)
         utils.xml_node_set(css, "priority", priority)
         utils.xml_node_set(css, "is_global", is_global)
 
-        # TODO: save CSS file if it has changes
+        if filename:
+            # Load from file
+            utils.xml_node_set(css, "filename", filename)
+
+            if os.path.isabs(filename):
+                fullpath = filename
+            elif self.filename:
+                dirname = os.path.dirname(self.filename)
+                fullpath = os.path.join(dirname, filename)
+
+            with open(fullpath, "w") as fd:
+                fd.write(css_text)
+
+                m = hashlib.sha256()
+                m.update(css_text.encode())
+
+                utils.xml_node_set(css, "sha256", m.hexdigest())
+        else:
+            # Load from project
+            content = E.content(css_text)
+            css.append(content)
 
         return css
 
     def __save_gresource_and_get_node(self, gresource_id, filename):
+        file_object = self.get_gresource_by_id(gresource_id)
         gresources = E.gresources()
 
         if filename:
             root = self.db.export_gresource(gresource_id)
-            self.__save_xml_and_update_node(gresources, root, filename)
+            self.__save_xml_and_update_node(gresources, root, filename, file_object)
         else:
             # Embed file contents in project as CDATA
             root = self.db.export_gresource(gresource_id)
@@ -720,10 +777,10 @@ class CmbProject(GObject.Object, Gio.ListModel):
             gresources = self.__save_gresource_and_get_node(gresource_id, gresources_filename)
             project.append(gresources)
 
-        # Save CSS reference
-        for row in c.execute("SELECT css_id, filename, priority, is_global FROM css;"):
-            css_id, css_filename, priority, is_global = row
-            css = self.__save_css_and_get_node(css_id, css_filename, priority, is_global)
+        # Save CSS files
+        for row in c.execute("SELECT css_id, filename, css, priority, is_global FROM css;"):
+            css_id, css_filename, css, priority, is_global = row
+            css = self.__save_css_and_get_node(css_id, css_filename, css, priority, is_global)
             project.append(css)
 
         # Save UI files
@@ -810,13 +867,17 @@ class CmbProject(GObject.Object, Gio.ListModel):
 
         self.history_push(_('Import file "{filename}"').format(filename=filename))
 
-        # Remove old UI
-        if overwrite:
-            self.db.execute("DELETE FROM ui WHERE filename=?;", (filename,))
-
-        # Import file
         self.foreign_keys = False
 
+        # Remove old UI
+        old_ui = None
+        if overwrite:
+            row = self.db.execute("DELETE FROM ui WHERE filename=? RETURNING ui_id;", (filename,)).fetchone()
+            ui_id, = row if row else (None, )
+            old_ui = self.get_object_by_id(ui_id)
+            self.__file_state.pop(filename)
+
+        # Import file
         if filename.endswith(".blp"):
             root, relpath, hexdigest = self.__parse_blp_file(filename)
         else:
@@ -828,6 +889,9 @@ class CmbProject(GObject.Object, Gio.ListModel):
         import_end = time.monotonic()
 
         # Populate UI
+        if old_ui:
+            self.__remove_ui(old_ui)
+
         self.__populate_ui(ui_id)
 
         self.history_pop()
@@ -876,7 +940,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 else:
                     continue
             except Exception as e:
-                print(e)
+                logger.warning(e)
                 continue
 
             template = root.find("template")
@@ -906,21 +970,35 @@ class CmbProject(GObject.Object, Gio.ListModel):
             ts = TopologicalSorter(ui_node_graph)
             sorted_ui_nodes = list(ts.static_order())
         except CycleError as e:
-            raise Exception(f"Could not load project because of dependency cycle {e}")
+            logger.warning(f"Dependency cycle detected: {e}")
+            raise Exception(_("Could not load project because of dependency cycle"))
 
         return sorted_ui_nodes
 
-    def import_gresource(self, filename):
+    def import_gresource(self, filename, overwrite=False):
         self.history_push(_('Import GResource "{filename}"').format(filename=filename))
 
+        # Remove old UI
+        old_gresource = None
+        if overwrite:
+            row = self.db.execute(
+                "DELETE FROM gresource WHERE resource_type='gresources' AND gresources_filename=? RETURNING gresource_id;",
+                (filename, )
+            ).fetchone()
+            gresource_id, = row if row else (None, )
+            old_gresource = self.get_gresource_by_id(gresource_id)
+            self.__file_state.pop(filename)
+
         root, relpath, hexdigest = self.__parse_xml_file(filename)
         gresource_id = self.db.import_gresource_from_node(root, relpath)
 
         # Populate UI
+        if old_gresource:
+            self.__remove_gresource(old_gresource)
         self.__populate_gresource(gresource_id)
 
         self.history_pop()
-        return gresource_id
+        return self.get_gresource_by_id(gresource_id)
 
     def __selection_remove(self, obj):
         if obj not in self.__selection:
@@ -1018,28 +1096,37 @@ class CmbProject(GObject.Object, Gio.ListModel):
         retval = []
         for row in c.execute("SELECT ui_id FROM ui ORDER BY ui_id;"):
             ui = self.get_object_by_id(row[0])
-            retval.append(ui)
+
+            if ui:
+                retval.append(ui)
 
         c.close()
         return retval
 
-    def __add_css(self, emit, css_id, filename=None, priority=None, is_global=None):
-        css = CmbCSS(project=self, css_id=css_id)
-        self.__css_id[css_id] = css
+    def __add_css(self, emit, css_id, filename=None, priority=None, is_global=None, css=None):
+        css_object = CmbCSS(project=self, css_id=css_id)
+        self.__css_id[css_id] = css_object
         if emit:
-            self.emit("css-added", css)
+            self.emit("css-added", css_object)
 
-        return css
+        return css_object
 
     def add_css(self, filename=None):
         basename, relpath = self._get_basename_relpath(filename)
 
         try:
             self.history_push(_("Add CSS {basename}").format(basename=basename or ""))
-            css_id = self.db.add_css(relpath)
+            if filename and os.path.exists(filename):
+                with open(filename, "r") as fd:
+                    css = fd.read()
+            else:
+                css = None
+
+            css_id = self.db.add_css(relpath, css=css)
             self.db.commit()
             self.history_pop()
         except Exception:
+            logger.warning("Tried to add CSS", exc_info=True)
             return None
         else:
             return self.__add_css(True, css_id, relpath)
@@ -1057,7 +1144,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
             self.db.commit()
             self.__remove_css(css)
         except Exception as e:
-            print(e)
+            logger.warning(e)
 
     def __add_gresource(
         self,
@@ -1266,7 +1353,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
 
     def remove_object(self, obj, allow_internal_removal=False):
         if not allow_internal_removal and obj.internal:
-            raise Exception("Internal objects can not be removed")
+            raise Exception(_("Internal objects can not be removed"))
 
         try:
             was_selected = obj in self.__selection
@@ -1407,7 +1494,6 @@ class CmbProject(GObject.Object, Gio.ListModel):
         p = properties.get(property_id, None)
 
         if p and p.owner_id == owner_id and p.property_id == property_id:
-            print("AAAAAAAAA", p, prop, property_id)
             p.notify(prop)
 
     def __undo_redo_do(self, undo, update_objects=None):
@@ -1650,6 +1736,21 @@ class CmbProject(GObject.Object, Gio.ListModel):
             if ui:
                 ui._library_changed(pk[1])
 
+    def __db_freeze(self):
+        self.history_enabled = False
+        self.db.foreign_keys = False
+        self.db.ignore_check_constraints = True
+
+    def __db_thaw(self):
+        self.db.foreign_keys = True
+        self.db.ignore_check_constraints = False
+        self.history_enabled = True
+
+    def clear_history(self):
+        self.history_index = 0
+        self.db.clear_history()
+        self.emit("changed")
+
     def __undo_redo(self, undo):
         selection = self.get_selection()
 
@@ -1660,9 +1761,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
         update_parents = []
 
         try:
-            self.history_enabled = False
-            self.db.foreign_keys = False
-            self.db.ignore_check_constraints = True
+            self.__db_freeze()
 
             if command == "POP":
                 if undo:
@@ -1683,16 +1782,10 @@ class CmbProject(GObject.Object, Gio.ListModel):
                 # Undo / Redo in DB
                 self.__undo_redo_do(undo, update_parents)
 
-            self.db.foreign_keys = True
-            self.db.ignore_check_constraints = False
-            self.history_enabled = True
+            self.__db_thaw()
         except sqlite3.Error as e:
-            self.history_enabled = True
-            self.db.foreign_keys = True
-            self.db.ignore_check_constraints = False
-
-            self.history_index = 0
-            self.db.clear_history()
+            self.__db_thaw()
+            self.clear_history()
             raise e
 
         # Compress update commands
@@ -2324,7 +2417,7 @@ class CmbProject(GObject.Object, Gio.ListModel):
             self.history_pop()
             self.db.commit()
         except Exception as e:
-            print(f"Error removing parent of object {obj} {e}")
+            logger.warning(f"Error removing parent of object {obj} {e}")
         finally:
             self.__remove_object(parent)
             if grand_parent is None:
@@ -2432,6 +2525,10 @@ class CmbProject(GObject.Object, Gio.ListModel):
 
         path_parent = item.path_parent
 
+        # Do not do anything if item has no parent
+        if not path_parent and not os.path.dirname(filename):
+            return
+
         # Do not do anything if the path is the same
         if path_parent and path_parent.path and path_parent.path == os.path.dirname(filename):
             return
