@@ -30,7 +30,7 @@ import time
 import atexit
 import socket
 
-gi.require_version('Casilda', '0.2')
+gi.require_version('Casilda', '1.0')
 from gi.repository import GObject, GLib, Gio, Gdk, Gtk, Casilda
 
 from . import config
@@ -163,21 +163,15 @@ class CmbMerengueProcess(GObject.Object):
             finally:
                 self.__pid = 0
 
-    def write_command(self, command, payload=None, args=None):
+    def write_command(self, command, args=None):
         cmd = {"command": command}
 
-        if payload is not None:
-            # Encode to binary first, before calculating lenght
-            payload = payload.encode()
-            cmd["payload_length"] = len(payload)
-            logger.debug(f"write_command {command} {len(payload)}")
-
         if args is not None:
             cmd["args"] = args
 
-        self.__socket_write_command(cmd, payload)
+        self.__socket_write_command(cmd)
 
-    def __socket_write_command(self, cmd, payload=None):
+    def __socket_write_command(self, cmd):
         # Send command in one line as json
 
         def write_data(data):
@@ -188,11 +182,6 @@ class CmbMerengueProcess(GObject.Object):
 
         write_data(json.dumps(cmd).encode())
         write_data(b"\n")
-
-        if payload is not None:
-            write_data(payload)
-
-        # Flush
         self.__command.flush()
 
     def __on_exit(self, pid, status, data):
@@ -253,11 +242,11 @@ class CmbView(Gtk.Box):
         self.__merengue_command("quit")
         self.__merengue.cleanup()
 
-    def __merengue_command(self, command, payload=None, args=None):
+    def __merengue_command(self, command, args=None):
         if not self.__merengue_started:
             return
 
-        self.__merengue.write_command(command, payload, args)
+        self.__merengue.write_command(command, args)
 
     def __get_ui_xml(self, ui_id, merengue=False):
         if self.show_merengue:
@@ -319,12 +308,12 @@ class CmbView(Gtk.Box):
 
         self.__merengue_command(
             "update_ui",
-            payload=ui,
             args={
                 "ui_id": ui_id,
                 "dirname": self.__get_ui_dirname(ui_id),
                 "toplevels": toplevels,
                 "selection": objects,
+                "xml": ui,
             },
         )
 
@@ -440,6 +429,9 @@ class CmbView(Gtk.Box):
         self.__merengue_command("remove_css_provider", args={"css_id": obj.css_id})
 
     def __on_css_changed(self, project, obj, field):
+        if field == "css":
+            return
+
         value = obj.get_property(field)
 
         if field == "filename" and value:
@@ -555,6 +547,9 @@ class CmbView(Gtk.Box):
         self.stack.props.visible_child_name = "ui_xml"
         self.__update_view()
 
+    def set_interactive_debugging(self, enable):
+        self.__merengue_command("set_interactive_debugging", args={"enable": enable})
+
     def restart_workspace(self):
         # Clear last exit timestamp
         self.__merengue_last_exit = None
@@ -573,6 +568,7 @@ class CmbView(Gtk.Box):
 
         retval.main_section.append(_("Restart workspace"), "win.workspace_restart")
         retval.main_section.append(_("Inspect UI definition"), "win.inspect")
+        retval.main_section.append(_("Gtk Inspector"), "win.open_inspector")
 
         return retval
 
@@ -589,6 +585,7 @@ class CmbView(Gtk.Box):
                 return
 
         self.__ui = None
+        self.__set_error_message(None)
         self.__merengue.start()
 
     def __command_selection_changed(self, selection):
@@ -684,6 +681,8 @@ class CmbView(Gtk.Box):
             if args["property"] == "gtk-theme-name":
                 self.__theme = args["value"]
                 self.notify("gtk_theme")
+        elif command == "update_ui_error":
+            self.__set_error_message(args["error"])
 
     def __add_remove_placeholder(self, command, modifier):
         if self.project is None:
