@@ -22,6 +22,7 @@
 
 import gi
 import importlib
+import logging
 
 # We need to use lxml to get access to nsmap
 from lxml import etree
@@ -43,13 +44,13 @@ class CmbGirData:
     def __init__(
         self,
         gir_file,
+        gtk_version=None,
         libname=None,
         types=None,
         flag_types=None,
         enum_types=None,
         boxed_types=[],
         skip_types=[],
-        target_gtk4=False,
         exclude_objects=False,
         external_types=None,
     ):
@@ -89,16 +90,18 @@ class CmbGirData:
         global nsmap
         nsmap = root.nsmap
 
+        self.lib = libname
+
         # Get <namespace/>
         namespace = root.find("namespace", nsmap)
 
         # Get module/name space data
         self.name = namespace.get("name")
-        self.prefix = namespace.get(ns("c", "identifier-prefixes"))
-        self.lib = libname
+
+        # Use first prefix by default, add new option if needed.
+        self.prefix = namespace.get(ns("c", "identifier-prefixes")).split(",")[0]
         self.version = namespace.get("version")
         self.shared_library = namespace.get("shared-library")
-        self.target_tk = "Gtk-4.0" if target_gtk4 else "Gtk+-3.0"
         self.accessibility_metadata = {}
 
         self.external_types = external_types if external_types else {}
@@ -113,7 +116,7 @@ class CmbGirData:
 
         # Load Module described by gir
         try:
-            print(f"Loading {self.name} {self.version}")
+            print(f"Loading {self.name} {self.version} with prefix {self.prefix}")
             gi.require_version(self.name, self.version)
             self.mod = importlib.import_module(f"gi.repository.{self.name}")
 
@@ -123,9 +126,9 @@ class CmbGirData:
                     init_function()
 
         except ValueError as e:
-            print(f"Oops! Could not load {self.name} {self.version} module: {e}")
+            logging.error(f"Could not load {self.name} {self.version} module: {e}")
 
-        gi.require_version("CmbCatalogUtils", "4.0" if target_gtk4 else "3.0")
+        gi.require_version("CmbCatalogUtils", "4.0" if gtk_version == 4 else "3.0")
 
         global CmbCatalogUtils
         from gi.repository import CmbCatalogUtils
@@ -148,9 +151,9 @@ class CmbGirData:
 
         self._cmb_types_init()
 
-        if target_gtk4:
+        if gtk_version == 4:
             self._gtk4_init()
-        else:
+        elif gtk_version == 3:
             self._gtk3_init()
 
         # Types dependency graph
@@ -192,6 +195,27 @@ class CmbGirData:
         else:
             InstanceClass = getattr(self.mod, name, None)
 
+        required_params = {}
+
+        # TODO: add this to the catalog directly
+        if name == "AdwLayoutSlot":
+            required_params = {
+                "id": "an slot id"
+            }
+        elif name == "AdwSwipeTracker":
+            from gi.repository import Adw
+            required_params = {
+                "swipeable": Adw.Flap(),
+            }
+        elif name == "AdwPropertyAnimationTarget":
+            class FooBar(GObject.GObject):
+                foobar = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE)
+
+            required_params = {
+                "object": FooBar(),
+                "pspec": FooBar.find_property("foobar")
+            }
+
         gtype = GObject.type_from_name(name)
         if InstanceClass is not None and GObject.type_is_a(gtype, GObject.GObject):
             if GObject.type_test_flags(gtype, GObject.TypeFlags.ABSTRACT):
@@ -200,9 +224,9 @@ class CmbGirData:
                     pass
 
                 if ChildClass is not None:
-                    retval = ChildClass()
+                    retval = ChildClass(**required_params)
             else:
-                retval = InstanceClass()
+                retval = InstanceClass(**required_params)
 
         # keep the instance for later
         if retval is not None:
@@ -304,9 +328,16 @@ class CmbGirData:
         if pspec.value_type == GObject.TYPE_BOOLEAN:
             return "True" if default_value != 0 else "False"
         elif GObject.type_is_a(pspec.value_type, GObject.TYPE_ENUM):
-            return CmbCatalogUtils.pspec_enum_get_default_nick(pspec.value_type, default_value)
+            return pspec.value_type.pytype(default_value).value_nick
         elif GObject.type_is_a(pspec.value_type, GObject.TYPE_FLAGS):
-            return CmbCatalogUtils.pspec_flags_get_default_nick(pspec.value_type, default_value)
+            none = pspec.value_type.pytype(0).first_value_nick
+            value_nicks = pspec.value_type.pytype(default_value).value_nicks
+
+            # Cleanup none value
+            if none in value_nicks:
+                value_nicks.remove(none)
+
+            return " | ".join(value_nicks) if value_nicks else None
         elif GObject.type_is_a(pspec.value_type, GObject.TYPE_GTYPE):
             return GObject.type_name(default_value) if default_value is not None else None
         elif GObject.type_is_a(pspec.value_type, GLib.strv_get_type()):
@@ -656,7 +687,7 @@ class CmbGirData:
 
                     if nick not in attr:
                         doc = member["doc"]
-                        print(f"Missing type value for {enumeration}:{nick} {doc}")
+                        logging.warning(f"Missing type value for {enumeration}:{nick} {doc}")
                         continue
 
             # Generate a list of properties for each enumeration member
@@ -721,7 +752,6 @@ class CmbGirData:
             type_name = self.pspec_map.get(pspec_type_name, None)
             if type_name is None:
                 self.ignored_pspecs.add(pspec_type_name)
-                self.ignored_types.add(type_name)
                 continue
 
             is_object = type_name == "object"
@@ -748,6 +778,7 @@ class CmbGirData:
             retval[name] = {
                 "type": type_name,
                 "is_object": is_object,
+                "disable_inline_object": is_object and type_name in ["GMenuModel"],
                 "version": child.get("version"),
                 "deprecated_version": child.get("deprecated-version"),
                 "construct": child.get("construct-only"),
@@ -783,6 +814,16 @@ class CmbGirData:
 
         return retval
 
+    def _list_properties(self, type_name):
+        nons_name = type_name.removeprefix(self.prefix)
+        klass = getattr(self.mod, nons_name)
+        gtype = klass.__gtype__
+
+        if gtype.is_a(GObject.Object) or gtype.is_interface():
+            return GObject.list_properties(klass)
+        else:
+            return []
+
     def _get_type_data(self, element, name, use_instance=True, skip_types=[]):
         parent = element.get("parent")
 
@@ -805,10 +846,6 @@ class CmbGirData:
         is_container = False
         overrides = []
 
-        nons_name = name.removeprefix(self.prefix)
-        GObject.type_ensure(getattr(self.mod, nons_name).__gtype__)
-        props = CmbCatalogUtils.get_class_properties(name)
-
         if use_instance:
             instance = self._get_instance_from_type(name)
             if instance is not None:
@@ -816,6 +853,8 @@ class CmbGirData:
                 if parent not in skip_types:
                     overrides = self._type_get_properties_overrides(name)
 
+        props = self._list_properties(name)
+
         return {
             "parent": parent,
             "layout": "container" if is_container else None,
@@ -905,9 +944,7 @@ class CmbGirData:
                 continue
 
             if types is None or name in types:
-                # NOTE: this method is needed because
-                # g_object_interface_list_properties bindings do not work
-                props = CmbCatalogUtils.get_iface_properties(name)
+                props = self._list_properties(name)
 
                 retval[name] = {
                     "parent": "interface",
@@ -1052,7 +1089,7 @@ class CmbGirData:
                         break
 
                 if row is None:
-                    print(f"Error trying to find {parent_owner}::{property_id} property definition")
+                    logging.error(f"Error trying to find {parent_owner}::{property_id} property definition")
                     break
 
                 (type_id, is_object, construct_only, minimum, maximum, version, deprecated_version,
